C
C       Decls of SFxxx functions for Fortran multi-file interface
C
      integer  sfstart
      external sfstart
      integer  sfn2index
      external sfn2index
      integer  sfcreate
      external sfcreate
      integer  sfsdmstr
      external sfsdmstr
      integer  sfsdmname
      external sfsdmname
      integer  sfsdtstr
      external sfsdtstr
      integer  sfgdtstr
      external sfgdtstr
      integer  sfgdmstr
      external sfgdmstr
      integer  sfginfo
      external sfginfo
      integer  sfgainfo
      external sfgainfo
      integer  sfgdinfo
      external sfgdinfo
      integer  sfsattr
      external sfsattr
      integer  sfsnatt
      external sfscatt
      integer  sfscatt
      integer  sffattr
      external sffattr
      integer  sfend
      external sfend
      integer  sfendacc
      external sfendacc
      integer  sffinfo
      external sffinfo
      integer  sfselect
      external sfselect
      integer  sfdimid
      external sfdimid
      integer  sfgcal
      external sfgcal
      integer  sfscal
      external sfscal
      integer  sfsdscale
      external sfsdscale
      integer  sfgdscale
      external sfgdscale
      integer  sfsfill
      external sfsfill
      integer  sfgfill
      external sfgfill
      integer  sfgrange
      external sfgrange
      integer  sfsrange
      external sfsrange
      integer  sfrattr
      external sfrattr
      integer  sfrnatt
      external sfrcatt
      integer  sfrcatt
      integer  sfrdata
      external sfrdata
      integer  sfwdata
      external sfwdata
      integer  sfsextf
      external sfsextf
      integer  sfsnbit
      external sfsnbit
      integer  sfsacct
      external sfsacct
      integer  sfid2ref
      external sfid2ref
      integer  sfiscvar
      external sfiscvar
      integer  sfref2index
      external sfref2index
      integer  sfsdmvc
      external sfsdmvc
      integer  sfisdmvc
      external sfisdmvc
      integer  sfgichnk
      external sfgichnk
      integer  sfrcchnk
      external sfrcchnk
      integer  sfrchnk
      external sfrchnk
      integer  sfwcchnk
      external sfwcchnk
      integer  sfwchnk
      external sfwchnk
      integer  sfscchnk
      external sfscchnk
      integer  sfschnk
      external sfschnk
      integer  sfscompress
      external sfscompress
      integer  sfgcompress
      external sfgcompress
      integer  sfisrcrd
      external sfisrcrd
      integer  sfsblsz
      external sfsblsz
      integer  sfchempty
      external sfchempty
      integer  sfwcdata
      external sfwcdata
      integer  sfrcdata
      external sfrcdata
      integer  sfgcfill
      external sfgcfill
      integer  sfscfill
      external sfscfill
      integer  sfsflmd
      external sfsflmd
      integer  sfgetfname
      external sfgetfname
      integer  sfgetnamelen
      external sfgetnamelen
      integer  sfidtype
      external sfidtype
      integer  sfgnvars_byname
      external sfgnvars_byname
      integer  sfn2indices
      external sfn2indices
      integer  sfgmaxopenf
      external sfgmaxopenf
      integer  sfgnumopenf
      external sfgnumopenf
      integer  sfrmaxopenf
      external sfrmaxopenf
      
