EXPORTS
; DFFF
DFISHDF
DFACCESS
DFOPEN
DFIISHDF
DFIACCESS
DFIOPEN

; DFR8FF
DFR8NIMAGES
DFR8WRITEREF
DFR8READREF
DFR8ADDIMAGE
DFR8PUTIMAGE
DFR8GETIMAGE
DFR8GETDIMS
D8WREF
D8NIMS
D8RREF
D8AIMG
D8PIMG
D8GIMG
D8GDIMS
D8INIMS
D8IWREF
D8IRREF
D8IAIMG
D8IPIMG
D8IGIMG
D8IGDIM

; DF24FF
DF24READREF
DF24PUTIMAGE
DF24ADDIMAGE
DF24GETIMAGE
DF24GETDIMS
D2NIMG
D2RREF
D2PIMG
D2AIMG
D2GIMG
D2GDIMS
D2IRREF
D2IAIMG
D2IGIMG
D2IGDIM
D2INIMG

; MFGRFF
MGGNLUTS
MGGCOMPRESS
MGSCOMPRESS
MGWCHNK
MGWCCHNK
MGSCHNK
MGSCCHNK
MGRCHNK
MGRCCHNK
MGGICHNK
MGFNDAT
MGSATTR
MGSNATT
MGSCATT
MGSXFIL
MGN2NDX
MGCREAT
MGCGNLUTS
MGCGCOMPRESS
MGCSCOMPRESS
MGCWCHNK
MGCWCCHNK
MGCSCHNK
MGCSCCHNK
MGCRCHNK
MGCRCCHNK
MGCGICHNK
MGIFNDAT
MGISATTR
MGISCATT
MGISXFIL
MGIN2NDX
MGICREAT

; DFUFP2IFF
DFUFPTOIMAGE
DUF2IM
DUIF2I

; DFPFF
DFPREADREF
DFPWRITEREF
DFPNPALS
DFPPUTPAL
DFPADDPAL
DFPGETPAL
DPRREF
DPWREF
DPNPALS
DPPPAL
DPAPAL
DPGPAL
DPIRREF
DPIWREF
DPINPAL
DPIPPAL
DPIGPAL

; HFILEFF
HCONFINF
HEPRNTF
HESTRINGF
HISHDFF
HGFILVER
HGLIBVER
HXSCDIR
HXSDIR
HOPEN
HCONFINFC
HEPRNTC
HESTRINGC
HIISHDF
HGFILVERC
HGLIBVERC
HXISCDIR
HXISDIR
HIOPEN

; DFSDFF
DSRSLAB
DSSSLAB
DSWREF
DSGDAST
DSGDIST
DFSDGETDIMSTRS
DFSDNUMBER
DFSDREADREF
DFSDSTARTSLICE
DFSDGETSLICE
DFSDADDDATA
DFSDPUTDATA
DFSDSETDIMSTRS
DFSDSETDATASTRS
DFSDGETDATA
DFSDGETDIMS
DSP32SD
DSNUM
DSRREF
DSSSLC
DSGSLC
DSADATA
DSPDATA
DSSDIST
DSSDAST
DSGDATA
DSGDIMS
DSIRSLAB
DSISSLAB
DSIWREF
DSIGDAS
DSIGDIS
DSINUM
DSIRREF
DSISSLC
DSIGSLC
DSIADAT
DSIPDAT
DSISDIS
DSISDAS
DSIGDAT
DSIGDIM
DSIP32S

; VATTRFF
VFFDATT
VFSCATT
VFSNATT
VSFAINF
VSFFDAT
VSFSCAT
VSFSNAT
VSFFIDX
VFCFDAT
VFCSCAT
VFCSATT
VSFCAIN
VSFCFDA
VSFCSCA
VSFCSAT
VSFCFDX

; VGFF
VFGVGROUPS
VSFGVDATAS
VSFGETBLINFO
VSFSETNMBL
VSFSETBLSZ
VFFNAME
VSFFCLS
VFDTR
VSFCPAK
VSFNPAK
VSQFNAME
VSQFFLDS
VFADTR
VFGTTR
VFGTTRS
VFNTR
VFINQTR
VFFLOC
VHFMKGP
VHFSCDM
VHFSCD
VHFSDM
VHFSD
VFNDCLS
VFIND
VSFLONE
VFLONE
VFENTS
VSFSIZ
VSFGFLD
VSFELTS
VSFGINT
VSFSEXTF
VSFWRIT
VSFREAD
VSFFDEF
VSFSINT
VSFSFLD
VSFSCLS
VSFSNAM
VSFDLTE
VSFGID
VSFFND
VSFEX
VSFINQ
VSFGCLS
VSFGNAM
VSFSEEK
VSFDTCH
VSFATCH
VFISVS
VFISVG
VFINSRT
VFSCLS
VFSNAM
VFGNXT
VFGID
VFINQ
VFGCLS
VFGNAM
VFDTCH
VFATCH
VSCGBLINFO
VSCSETNMBL
VSCSETBLSZ
VFFNAMEC
VSCFCLS
VDTRC
VSFCCPK
VSFNCPK
VSQNAMEC
VSQFLDSC
VADTRC
VGTTRC
VGTTRSC
VNTRC
VINQTRC
VFLOCC
VHMKGPC
VHSCDMC
VHSCDC
VHSDMC
VHSDC
VFNDCLSC
VFINDC
VSLONEC
VLONEC
VENTSC
VSSIZC
VSGFLDC
VSELTSC
VSGINTC
VSSEXTFC
VSWRITC
VSREADC
VSFDEFC
VSSINTC
VSSFLDC
VSSCLSC
VSSNAMC
VSDLTC
VSGIDC
VSFNDC
VSFEXC
VSINQC
VSGCLSC
VSGNAMC
VSSEEKC
VSDTCHC
VSATCHC
VISVSC
VISVGC
VINSRTC
VSCLSC
VSNAMC
VGNXTC
VGIDC
VINQC
VGCLSC
VGNAMC
VDTCHC
VATCHC

; DFANFF
DFANADDFID
DFANLABLIST
DFANPUTDESC
DFANPUTLABEL
DFANGETDESC
DFANGETDESCLEN
DFANGETLABEL
DFANGETLABLEN
DAAFID
DALLIST
DAPDESC
DAPLAB
DAGDESC
DAGDLEN
DAGLAB
DAGLLEN
DAIAFID
DAILIST
DAIPANN
DAIGANN
DAIGANL
