EXPORTS

; MFSDFF
SFRMAXOPENF
SFGNUMOPENF
SFGMAXOPENF
SFN2INDICES
SFGNVARS_BYNAME
SFIDTYPE
SFGETNAMELEN
SFGETFNAME
SFCHEMPTY
SFGCOMPRESS
SFSCOMPRESS
SFWCHNK
SFWCCHNK
SFSCHNK
SFSCCHNK
SFRCHNK
SFRCCHNK
SFGICHNK
SFSEXTF
SFFATTR
SFSATTR
SFSNATT
SFSCATT
SFGDINFO
SFGAINFO
SFGINFO
SFGDMSTR
SFGDTSTR
SFSDTSTR
SFSDMNAME
SFSDMSTR
SFCREATE
SFN2INDEX
SFSTART
SCRMAXOPENF
SCGNUMOPENF
SCGMAXOPENF
SCN2INDICES
SCGNVARS_BYNAME
SCIDTYPE
SCGETNAMELEN
SCGETFNAME
SCCHEMPTY
SCGCOMPRESS
SCSCOMPRESS
SCWCHNK
SCWCCHNK
SCSCHNK
SCSCCHNK
SCRCHNK
SCRCCHNK
SCGICHNK
SCSEXTF
SCFATTR
SCSATTR
SCSNATT
SCSCATT
SCGDINFO
SCGAINFO
SCGINFO
SCGDIMSTRS
SCGDATSTRS
SCSDATSTR
SCSDIMNAME
SCSDIMSTR
SCCREATE
SCN2INDEX
SCSTART
