/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of HDF.  The full HDF copyright notice, including       *
 * terms governing use, modification, and redistribution, is contained in    *
 * the files COPYING and Copyright.html.  COPYING can be found at the root   *
 * of the source code distribution tree; Copyright.html can be found at      *
 * http://hdfgroup.org/products/hdf4/doc/Copyright.html.  If you do not have *
 * access to either file, you may request a copy from help@hdfgroup.org.     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* $Id: tproto.h 5692 2011-09-19 16:05:25Z bmribler $ */

#ifndef _TPROTO_H
#define _TPROTO_H

#include "hdf.h"
#include "tutils.h"

void        test_an(void);
void        test_an_2(void);
void        test_anfile(void);
void        test_man(void);
void        test_hblocks(void);
void        test_hextelt(void);
void        test_hfile(void);
void        test_hfile1(void);
void        test_r24(void);
void        test_GR(void);
void        test_r8(void);
void        test_pal(void);
void        test_sdmms(void);
void        test_sdnmms(void);
void        test_tsdstr(void);
void        test_vers(void);
void        test_slab(void);
void        test_litend(void);
void        test_vsets(void);
void        test_vnameclass(void);
void        test_vset_attr(void);
void        test_vspack(void);
void        test_comp(void);
void        test_bitio(void);
void        test_tbbt(void);
void        test_macros(void);
void        test_conv(void);
void        test_nbit(void);
void        test_mgr(void);
void        test_mgr_attr(void);
void        test_mgr_compress(void);
void        test_bitvect(void);
void        test_chunks(void);
void        test_buffer(void);
void        test_datainfo(void);
void        test_datainfo(void);
void        test_attdatainfo(void);

#endif /* _TPROTO_H */
